vec2 nearestNeighbor(vec2 loc, vec2 size) {
    return (floor(loc * size) + 0.5) / size;
}

void main()
{
    vec2 texCoord = vec2(v_tex_coord.x - 0.5, 1.0 - v_tex_coord.y - 0.5);
    
    float px = texCoord.x;
    float py = texCoord.y + u_focal_lenght;
    float pz = texCoord.y + 0.5; // u_horizon
    
    // projection from a 3d point to a 2d screenpixel; y stands for z (depth)
    float space_x = px / pz;
    float space_y = py / pz * -1;
    
    // invert the y direction so that everthing points in a correct way
    // a trigonomic solution to be able to rotate the bmap so you can look around 360°
    float screen_x = space_x * u_angle_cos - space_y * u_angle_sin;
    float screen_y = space_x * u_angle_sin + space_y * u_angle_cos;
    
    // final transformation and scaling
    texCoord = vec2(screen_x * u_scale + u_offset.x, screen_y * u_scale + u_offset.y);
    texCoord = nearestNeighbor(texCoord, u_texture_size);
    
    vec4 texColor;
    if (texCoord.x >= 0.0 && texCoord.x <= 1.0 && texCoord.y >= 0.0 && texCoord.y <= 1.0)
        texColor = texture2D(u_texture, texCoord);
    else {
        // repeat texture
        texCoord = mod(texCoord, u_repeat_factor) / u_repeat_factor;
        texColor = texture2D(u_repeat_texture, texCoord);
    }
    
    gl_FragColor = texColor * v_color_mix.a;
}
